==> pnl-0.1.0.tar.gz: pnl/log/pnlog.c <==
#ifdef __cplusplus
extern "C" {
#endif
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <sys/time.h>
#include <time.h>
#ifdef __cplusplus
}
#endif

#define FALSE	(0)
#define TRUE	(1)

#define FAIL	(-1)
#define SUCC	(0)

static int log_enabled=FALSE;
static int debug_enabled=TRUE;
static char *log_dir=NULL;
static char *log_prefix=NULL;
static char *log_extension=NULL;
static FILE *log_fp=NULL;

static int time_format(char *buf, size_t bufsiz, char *format, time_t t)
{
	struct tm t2; 
	
	(void)localtime_r (&t, &t2); 
	
	return ( (strftime (buf, bufsiz, format, &t2)) > 0 ? 1 : 0 );
}


int pnlog_hexwrite(const char *tag, FILE **wfp, const void *pp, int length);

/*!
 * Enable
 */
void pnlog_enable(void)
{
	log_enabled = TRUE;
}

/*!
 * Disable
 */
void pnlog_disable(void)
{
	log_enabled = FALSE;
}

/*!
 * Enable Debug
 */
void pnlog_enable_debug(void)
{
	debug_enabled = TRUE;
}

/*!
 * Diable Debug
 */
void pnlog_disable_debug(void)
{
	debug_enabled = FALSE;
}


/*!
 * Must call this function when you don't need anymore TRCLOG.
 */
void pnlog_close(void)
{
	pnlog_disable();
	if(log_dir) free(log_dir);
	if(log_prefix) free(log_prefix);
	if(log_extension) free(log_extension);
	if (log_fp) fclose(log_fp);
}

/*!
 *    Լ ȣؾ Ѵ.
 * @param dir αװ  丮 Ѵ.
 * @param prefix α ξ Ѵ.
 * @param extension α Ȯڸ Ѵ.
 */
int pnlog_init(const char *dir, 
				const char *prefix, 
				const char *extension)
{
	/* Fix Log Directory */
	log_dir = strdup(dir);
	if(!log_dir) {
		fprintf(stderr, "%s:%d:strdup error", __FILE__, __LINE__);
		return FAIL;
	}
	/* Fix LogFile Prefix */
	log_prefix = strdup(prefix);
	if(!log_prefix) {
		fprintf(stderr, "%s:%d:strdup error", __FILE__, __LINE__);
		return FAIL;
	}
	/* Fix LogFile Extension */
	log_extension = strdup(extension);
	if(!log_extension) {
		fprintf(stderr, "%s:%d:strdup error", __FILE__, __LINE__);
		return FAIL;
	}

	return SUCC;
}



/*
 * Log  Լ
 */
int pnlog_write(FILE **wfp, 
				const char *alert_msg, 
				const char *fmt, 
				va_list args)
{
	static char fname[1024]="";
	char tmp[1024];
	char file[1024];
	char now[60];
	char yyyymmdd[60];
	struct timeval tv;
	struct timezone tz;

	if (!log_enabled) return 0;

	/*  ¥ Ѵ */
	memset(yyyymmdd, 0x00, sizeof(yyyymmdd));
	time_format(yyyymmdd, sizeof(yyyymmdd), "%Y%m%d", time(NULL));

	/* ξ, Ȯڿ Ѵ.  */
	memset(tmp, 0x00, sizeof(tmp));
	sprintf(tmp, "%s%s%s", log_prefix, yyyymmdd, log_extension);

	/* ó Ȥ ¥ Ʋ( Ǹ)   ݰ  . */
	if ( strcmp(tmp, fname) ) {
		if(*wfp) fclose(*wfp);
		strcpy(fname, tmp);
		memset(file, 0x00, sizeof(file));
		sprintf(file, "%s/%s", log_dir, fname);
		if ( (*wfp = fopen(file, "a+")) == NULL) {
			fprintf(stderr, "fopen error:%s", file);
			return FAIL;
		}
	}

	/*  ð Ѵ. */
    memset(now, 0x00, sizeof(now));
   	gettimeofday(&tv, &tz);
   	time_format(now, sizeof(now), "%Y/%m/%d %H:%M:%S", tv.tv_sec);
   	sprintf(now, "%s.%03d ", now, (int)(tv.tv_usec / 1000));
   	fprintf( *wfp, "%s", now);

	/* ERROR, WARNING  Ѵ. */
	fprintf( *wfp, "%12s ", alert_msg);

	/* ޽ Ѵ. */
	vfprintf( *wfp, fmt, args );

   	/* putc('\n', wfp);  */

	fflush(*wfp);

	return SUCC;
}

/*
 * ٴ α׸ Ѵ.
 */
inline int pnlog_writeln(FILE **wfp, 
				const char *alert_msg, 
				const char *fmt, 
				va_list args)
{
	int ret;

	if (!log_enabled) return 0;

	ret = pnlog_write(wfp, alert_msg, fmt, args);
	if(ret==SUCC) {
		fputc('\n', *wfp);
		fflush(*wfp);
	}

	return ret;
}

/*!
 * FATAL ERROR αױ
 */
int pnlog_fatal(const char *fmt, ...)
{
	int ret;
	va_list args;

	va_start (args, fmt);

	ret = pnlog_writeln(&log_fp, "FATAL ERROR", fmt, args);

	va_end (args);

	return ret;
}

/*!
 * ERROR αױ
 */
int pnlog_error(const char *fmt, ...)
{
	int ret;
	va_list args;

	va_start (args, fmt);

	ret = pnlog_writeln(&log_fp, "ERROR", fmt, args);

	va_end (args);

	return ret;
}

/*! 
 * WARNING αױ 
 */
int pnlog_warn(const char *fmt, ...)
{
	int ret;
	va_list args;

	va_start (args, fmt);

	ret = pnlog_writeln(&log_fp, "WARNING", fmt, args);

	va_end (args);

	return ret;

}

/*!
 * DEBUG αױ 
 */
int pnlog_debug(const char *fmt, ...)
{
	int ret;
	va_list args;

	if ( !debug_enabled ) return 0;

	va_start (args, fmt);

	ret = pnlog_writeln(&log_fp, "DEBUG", fmt, args);

	va_end (args);

	return ret;
}

/*!
 * TRACE αױ
 */
int pnlog_trace(const char *fmt, ...)
{
	int ret;
	va_list args;

	va_start (args, fmt);

	ret = pnlog_writeln(&log_fp, "TRACE", fmt, args);

	va_end (args);

	return ret;
}

/*!
 * Ϲ α .
 */
int pnlog_info(const char *fmt, ...)
{
	int ret;
	va_list args;

	va_start (args, fmt);

	ret = pnlog_writeln(&log_fp, "INFO", fmt, args);

	va_end (args);

	return ret;
}


int pnlog_hexwrite(const char *tag, FILE **wfp, const void *pp, int length)
{
	int i;
	static char fname[1024]="";
	unsigned char *ch;
	char tmp[1024];
	char file[1024];
	char now[60];
	char yyyymmdd[60];
	struct timeval tv;
	struct timezone tz;

	if (!log_enabled) return 0;

	/*  ¥ Ѵ */
	memset(yyyymmdd, 0x00, sizeof(yyyymmdd));
	time_format(yyyymmdd, sizeof(yyyymmdd), "%Y%m%d", time(NULL));

	/* ξ, Ȯڿ Ѵ.  */
	memset(tmp, 0x00, sizeof(tmp));
	sprintf(tmp, "%s%s%s", log_prefix, yyyymmdd, log_extension);

	/* ó Ȥ ¥ Ʋ( Ǹ)   ݰ  . */
	if ( strcmp(tmp, fname) ) {
		if(*wfp) fclose(*wfp);
		strcpy(fname, tmp);
		memset(file, 0x00, sizeof(file));
		sprintf(file, "%s/%s", log_dir, fname);
		if ( (*wfp = fopen(file, "a+")) == NULL) {
			fprintf(stderr, "fopen error:%s", file);
			return FAIL;
		}
	}

	/*  ð Ѵ. */
    memset(now, 0x00, sizeof(now));
   	gettimeofday(&tv, &tz);
   	time_format(now, sizeof(now), "%Y/%m/%d %H:%M:%S", tv.tv_sec);
   	sprintf(now, "%s.%03d ", now, (int)(tv.tv_usec / 1000));
   	fprintf( *wfp, "%s", now);

	/* ERROR, WARNING  Ѵ. */
	fprintf( *wfp, "%12s ", tag );

	/* ޽ Ѵ. */
	ch = (unsigned char *)pp;

	for ( i = 1 ; i <= length ; i++ ) {
		fprintf( *wfp, "%02x ", *(ch++));
		if ( (i % 10) == 0 ) {
			fputc( '\n', *wfp );
			fprintf(*wfp, "                                     ");
		}
	}
	fputc( '\n', *wfp );

	fflush(*wfp);

	return SUCC;

}

int pnlog_hex(const char *pp, size_t length)
{
	int ret;

	ret = pnlog_hexwrite("HEX CODE", &log_fp, pp, length);

	return ret;
}

/*
 * pnlog_hex with Tag
 */
int pnlog_hex_t(const char *tag, const char *pp, size_t length)
{
	int ret;

	ret = pnlog_hexwrite(tag, &log_fp, pp, length);

	return ret;
}

==> pnl-0.2.0.tar.gz: pnl/log/pnlog.c <==
#ifdef __cplusplus
extern "C" {
#endif
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <sys/time.h>
#include <time.h>
#ifdef __cplusplus
}
#endif

#define FALSE	(0)
#define TRUE	(1)

#define FAIL	(-1)
#define SUCC	(0)

static int log_enabled=FALSE;
static int debug_enabled=TRUE;
static char *log_dir=NULL;
static char *log_prefix=NULL;
static char *log_extension=NULL;
static FILE *log_fp=NULL;
static char fname[1024]="";

static int g_line;
static char g_srcfile[FILENAME_MAX];
static char g_logtype[80];

static int time_format(char *buf, size_t bufsiz, char *format, time_t t)
{
	struct tm t2; 
	
	(void)localtime_r (&t, &t2); 
	
	return ( (strftime (buf, bufsiz, format, &t2)) > 0 ? 1 : 0 );
}


int pnlog_hexwrite(const char *tag, FILE **wfp, const void *pp, int length);

/*!
 * Enable
 */
void pnlog_enable(void)
{
	log_enabled = TRUE;
}

/*!
 * Disable
 */
void pnlog_disable(void)
{
	log_enabled = FALSE;
}

/*!
 * Enable Debug
 */
void pnlog_enable_debug(void)
{
	debug_enabled = TRUE;
}

/*!
 * Diable Debug
 */
void pnlog_debug_disable(void)
{
	debug_enabled = FALSE;
}


/*!
 * Must call this function when you don't need anymore TRCLOG.
 */
void pnlog_close(void)
{
	pnlog_disable();
	if(log_dir) free(log_dir);
	if(log_prefix) free(log_prefix);
	if(log_extension) free(log_extension);
	if (log_fp) fclose(log_fp);
}

/*!
 * you first have to call this function.
 * @param dir indicate where log places.
 * @param prefix indicate prefix.
 * @param extension indicate extension.
 */
int pnlog_init(const char *dir, 
				const char *prefix, 
				const char *extension)
{
	/* Fix Log Directory */
	log_dir = strdup(dir);
	if(!log_dir) {
		fprintf(stderr, "%s:%d:strdup error", __FILE__, __LINE__);
		return FAIL;
	}
	/* Fix LogFile Prefix */
	log_prefix = strdup(prefix);
	if(!log_prefix) {
		fprintf(stderr, "%s:%d:strdup error", __FILE__, __LINE__);
		return FAIL;
	}
	/* Fix LogFile Extension */
	log_extension = strdup(extension);
	if(!log_extension) {
		fprintf(stderr, "%s:%d:strdup error", __FILE__, __LINE__);
		return FAIL;
	}

	return SUCC;
}



/**
 * Log write
 */
int pnlog_write(FILE **wfp, 
				const char *alert_msg, 
				const char *fmt, 
				va_list args)
{
	char tmp[1024];
	char file[1024];
	char now[60];
	char yyyymmdd[60];
	struct timeval tv;
	struct timezone tz;

	if (!log_enabled) return 0;

	/*  ¥ Ѵ */
	memset(yyyymmdd, 0x00, sizeof(yyyymmdd));
	time_format(yyyymmdd, sizeof(yyyymmdd), "%Y%m%d", time(NULL));

	/* ξ, Ȯڿ Ѵ.  */
	memset(tmp, 0x00, sizeof(tmp));
	sprintf(tmp, "%s%s%s", log_prefix, yyyymmdd, log_extension);

	/* ó Ȥ ¥ Ʋ( Ǹ)   ݰ  . */
	if ( strcmp(tmp, fname) ) {
		if(*wfp) fclose(*wfp);
		strcpy(fname, tmp);
		memset(file, 0x00, sizeof(file));
		sprintf(file, "%s/%s", log_dir, fname);
		if ( (*wfp = fopen(file, "a+")) == NULL) {
			fprintf(stderr, "fopen error:%s", file);
			return FAIL;
		}
	}

	/*  ð Ѵ. */
    memset(now, 0x00, sizeof(now));
   	gettimeofday(&tv, &tz);
   	time_format(now, sizeof(now), "%Y/%m/%d %H:%M:%S", tv.tv_sec);
   	sprintf(now, "%s.%03d ", now, (int)(tv.tv_usec / 1000));
   	fprintf( *wfp, "%s", now);

	/* ERROR, WARNING  Ѵ. */
	fprintf( *wfp, "%12s ", alert_msg);

	/* ޽ Ѵ. */
	vfprintf( *wfp, fmt, args );

   	/* putc('\n', wfp);  */

	fflush(*wfp);

	return SUCC;
}

/**
 * write log line by line.
 */
inline int pnlog_writeln(FILE **wfp, 
				const char *alert_msg, 
				const char *fmt, 
				va_list args)
{
	int ret;

	if (!log_enabled) return 0;

	ret = pnlog_write(wfp, alert_msg, fmt, args);
	if(ret==SUCC) {
		fputc('\n', *wfp);
		fflush(*wfp);
	}

	return ret;
}

/*!
 * write log of FATAL ERROR.
 */
int pnlog_fatal(const char *fmt, ...)
{
	int ret;
	va_list args;

	va_start (args, fmt);

	ret = pnlog_writeln(&log_fp, "FATAL ERROR", fmt, args);

	va_end (args);

	return ret;
}

/*!
 * error log
 */
int pnlog_error(const char *fmt, ...)
{
	int ret;
	va_list args;

	va_start (args, fmt);

	ret = pnlog_writeln(&log_fp, "ERROR", fmt, args);

	va_end (args);

	return ret;
}

/*! 
 * WARNING log
 */
int pnlog_warn(const char *fmt, ...)
{
	int ret;
	va_list args;

	va_start (args, fmt);

	ret = pnlog_writeln(&log_fp, "WARNING", fmt, args);

	va_end (args);

	return ret;

}

/*!
 * DEBUG log
 */
int pnlog_debug(const char *fmt, ...)
{
	int ret;
	va_list args;

	if ( !debug_enabled ) return 0;

	va_start (args, fmt);

	ret = pnlog_writeln(&log_fp, "DEBUG", fmt, args);

	va_end (args);

	return ret;
}

/*!
 * TRACE log
 */
int pnlog_trace(const char *fmt, ...)
{
	int ret;
	va_list args;

	va_start (args, fmt);

	ret = pnlog_writeln(&log_fp, "TRACE", fmt, args);

	va_end (args);

	return ret;
}

/*!
 * INFO log
 */
int pnlog_info(const char *fmt, ...)
{
	int ret;
	va_list args;

	va_start (args, fmt);

	ret = pnlog_writeln(&log_fp, "INFO", fmt, args);

	va_end (args);

	return ret;
}


int pnlog_hexwrite(const char *tag, FILE **wfp, const void *pp, int length)
{
	int i;
	unsigned char *ch;
	char tmp[1024];
	char file[1024];
	char now[60];
	char yyyymmdd[60];
	struct timeval tv;
	struct timezone tz;

	if (!log_enabled) return 0;

	/*  ¥ Ѵ */
	memset(yyyymmdd, 0x00, sizeof(yyyymmdd));
	time_format(yyyymmdd, sizeof(yyyymmdd), "%Y%m%d", time(NULL));

	/* ξ, Ȯڿ Ѵ.  */
	memset(tmp, 0x00, sizeof(tmp));
	sprintf(tmp, "%s%s%s", log_prefix, yyyymmdd, log_extension);

	/* ó Ȥ ¥ Ʋ( Ǹ)   ݰ  . */
	if ( strcmp(tmp, fname) ) {
		if(*wfp) fclose(*wfp);
		strcpy(fname, tmp);
		memset(file, 0x00, sizeof(file));
		sprintf(file, "%s/%s", log_dir, fname);
		if ( (*wfp = fopen(file, "a+")) == NULL) {
			fprintf(stderr, "fopen error:%s", file);
			return FAIL;
		}
	}

	/*  ð Ѵ. */
    memset(now, 0x00, sizeof(now));
   	gettimeofday(&tv, &tz);
   	time_format(now, sizeof(now), "%Y/%m/%d %H:%M:%S", tv.tv_sec);
   	sprintf(now, "%s.%03d ", now, (int)(tv.tv_usec / 1000));
   	fprintf( *wfp, "%s", now);

	/* ERROR, WARNING  Ѵ. */
	fprintf( *wfp, "%12s ", tag );

	/* ޽ Ѵ. */
	ch = (unsigned char *)pp;

	for ( i = 1 ; i <= length ; i++ ) {
		fprintf( *wfp, "%02x ", *(ch++));
		if ( (i % 10) == 0 ) {
			fputc( '\n', *wfp );
			fprintf(*wfp, "                                     ");
		}
	}
	fputc( '\n', *wfp );

	fflush(*wfp);

	return SUCC;

}

int pnlog_hex(const char *pp, size_t length)
{
	int ret;

	ret = pnlog_hexwrite("HEX CODE", &log_fp, pp, length);

	return ret;
}

/*
 * pnlog_hex with Tag
 */
int pnlog_hex_t(const char *tag, const char *pp, size_t length)
{
	int ret;

	ret = pnlog_hexwrite(tag, &log_fp, pp, length);

	return ret;
}

void pnlog_locate(const char *file, const int line, const char *log_type)
{
	strcpy(g_srcfile, file);
	strcpy(g_logtype, log_type);
	g_line = line;
}

int pnlog_print(const char *fmt, ...)
{
	va_list args;
	char yyyymmdd[60];
	char now[60];
	char tmp[FILENAME_MAX];
	char file[FILENAME_MAX];
	struct timeval tv;
#ifndef _WIN32
	struct timezone tz; 
#endif

	va_start(args, fmt);

	/* get current date time */
	memset(yyyymmdd, 0x00, sizeof(yyyymmdd));
	time_format(yyyymmdd, sizeof(yyyymmdd), "%Y%m%d", time(NULL));

	/* write current date time */
    memset(now, 0x00, sizeof(now));
	memset(&tv, 0x00, sizeof(tv));
#ifndef _WIN32
   	gettimeofday(&tv, &tz);
#else
	tv.tv_sec = time(NULL);
#endif
   	time_format(now, sizeof(now), "%Y/%m/%d %H:%M:%S", tv.tv_sec);
#ifndef _WIN32
   	sprintf(now, "%s.%03d ", now, (int)(tv.tv_usec / 1000));
#endif

	memset(tmp, 0x00, sizeof(tmp));
	sprintf(tmp, "%s%s%s", log_prefix, yyyymmdd, log_extension); 
	if ( strcmp(tmp, fname) ) {
		if(log_fp) fclose(log_fp);
		strcpy(fname, tmp);
		memset(file, 0x00, sizeof(file));
		sprintf(file, "%s/%s", log_dir, fname);
		if ( (log_fp = fopen(file, "a+")) == NULL) {
			fprintf(stderr, "fopen error:%s", file);
			return FAIL;
		}
	}



   	fprintf( log_fp, "%s", now);

	/* write log type */
	fprintf( log_fp, "%12s ", g_logtype);

	/* write message */
	fprintf( log_fp, "%s:%d:", g_srcfile, g_line);
	vfprintf( log_fp, fmt, args );
	fputc('\n', log_fp);
	fflush(log_fp);

	va_end(args);

	return 0;
}
